'
' Copyright (c) 2012, MindFusion LLC - Bulgaria.
'

Imports System.Collections.Generic
Imports System.IO
Imports System.ComponentModel
Imports MindFusion.Diagramming

Imports MindFusion.Diagramming.Components
Imports MindFusion.Diagramming.Layout


Public Class EmployeeNode
    Inherits CompositeNode

    Shared Sub New()
        Images = New List(Of Image)(New Image() {My.Resources.E1, My.Resources.E2, My.Resources.E3, My.Resources.E4, My.Resources.E5, My.Resources.E6, _
         My.Resources.E7, My.Resources.E8})

        Titles = New List(Of String)(New String() {"Support", "Public Relations", "CEO", "Delivery", "Research", "Management", _
         "Development", "Consulting"})

        Fills = New List(Of MindFusion.Drawing.Brush)(New MindFusion.Drawing.Brush() {New MindFusion.Drawing.LinearGradientBrush(Color.White, Color.PaleGoldenrod, 180), New MindFusion.Drawing.LinearGradientBrush(Color.White, Color.PaleGreen, 180), New MindFusion.Drawing.LinearGradientBrush(Color.White, Color.PaleTurquoise, 180), New MindFusion.Drawing.LinearGradientBrush(Color.White, Color.PaleVioletRed, 180), New MindFusion.Drawing.LinearGradientBrush(Color.White, Color.PapayaWhip, 180), New MindFusion.Drawing.LinearGradientBrush(Color.White, Color.PeachPuff, 180), _
         New MindFusion.Drawing.LinearGradientBrush(Color.White, Color.Peru, 180), New MindFusion.Drawing.LinearGradientBrush(Color.White, Color.Pink, 180)})
    End Sub

    Public Sub New(ByVal diagram As Diagram)
        MyBase.New(diagram)
        Load()

        m_fullName.TextColor = Color.Gray
		'm_fullName.Text = Nothing
        m_text.TextColor = Color.Gray
		'm_text.Text = Nothing
		m_title.Text = ""

        Stroke = New MindFusion.Drawing.Pen(Color.Gray, 1)
        Fill = New MindFusion.Drawing.LinearGradientBrush(Color.White, Color.PaleGoldenrod, 0)
        Index = 0
	End Sub

    Public Sub New(ByVal prototype As EmployeeNode)
        MyBase.New(prototype)
		Load()

		m_fullName.TextColor = Color.Gray
		m_text.TextColor = Color.Gray

        Title = prototype.Title
        FullName = prototype.FullName
        Text = prototype.Text
        Stroke = prototype.Stroke
        Fill = prototype.Fill
        Index = prototype.Index
    End Sub

	Private Sub Load()
		Components.Clear()
		Components.Add(XmlLoader.Load(Content, Me, Nothing))

		shape = TryCast(FindComponent("Shape"), ShapeComponent)
		m_image = TryCast(FindComponent("Image"), ImageComponent)
		m_title = TryCast(FindComponent("Title"), TextComponent)
		m_fullName = TryCast(FindComponent("FullName"), EditComponent)
		AddHandler m_fullName.GotFocus, AddressOf OnFullNameGotFocus
		AddHandler m_fullName.LostFocus, AddressOf OnFullNameLostFocus
		AddHandler m_fullName.MouseMove, AddressOf OnFullNameMouseMove
		m_text = TryCast(FindComponent("Text"), EditComponent)
		AddHandler m_text.GotFocus, AddressOf OnTextGotFocus
		AddHandler m_text.LostFocus, AddressOf OnTextLostFocus
		AddHandler m_text.MouseMove, AddressOf OnTextMouseMove
	End Sub

    ' do not use the built-in component serialization
    Protected Overrides Function SerializeComponents() As Boolean
        Return False
    End Function

    Protected Overrides Sub SaveTo(ByVal writer As BinaryWriter, ByVal context As PersistContext)
        MyBase.SaveTo(writer, context)

        writer.Write(Index)
        writer.Write(Title)
        writer.Write(FullName)
        writer.Write(Text)
        writer.Write(m_fullName.TextColor.ToArgb())
        writer.Write(m_text.TextColor.ToArgb())

        Dim penConverter As TypeConverter = TypeDescriptor.GetConverter(GetType(MindFusion.Drawing.Pen))
        If penConverter IsNot Nothing AndAlso penConverter.CanConvertTo(GetType(String)) Then
            writer.Write(TryCast(penConverter.ConvertTo(Stroke, GetType(String)), String))
        End If

        Dim brushConverter As TypeConverter = TypeDescriptor.GetConverter(GetType(MindFusion.Drawing.Brush))
        If brushConverter IsNot Nothing AndAlso brushConverter.CanConvertTo(GetType(String)) Then
            writer.Write(TryCast(brushConverter.ConvertTo(Fill, GetType(String)), String))
        End If
    End Sub

    Protected Overrides Sub LoadFrom(ByVal reader As BinaryReader, ByVal context As PersistContext)
        MyBase.LoadFrom(reader, context)

        Index = reader.ReadInt32()
        Title = reader.ReadString()
        FullName = reader.ReadString()
        Text = reader.ReadString()
        m_fullName.TextColor = Color.FromArgb(reader.ReadInt32())
        m_text.TextColor = Color.FromArgb(reader.ReadInt32())

        Dim penConverter As TypeConverter = TypeDescriptor.GetConverter(GetType(MindFusion.Drawing.Pen))
        If penConverter IsNot Nothing AndAlso penConverter.CanConvertFrom(GetType(String)) Then
            Stroke = TryCast(penConverter.ConvertFrom(reader.ReadString()), MindFusion.Drawing.Pen)
        End If

        Dim brushConverter As TypeConverter = TypeDescriptor.GetConverter(GetType(MindFusion.Drawing.Brush))
        If brushConverter IsNot Nothing AndAlso brushConverter.CanConvertFrom(GetType(String)) Then
            Fill = TryCast(brushConverter.ConvertFrom(reader.ReadString()), MindFusion.Drawing.Brush)
        End If
    End Sub

    Private Sub OnPreviousImageClicked(ByVal sender As Object, ByVal e As EventArgs)
        If Index <> -1 Then
            Index = (Index - 1 + Images.Count) Mod Images.Count
        End If
    End Sub

    Private Sub OnNextImageClicked(ByVal sender As Object, ByVal e As EventArgs)
        If Index <> -1 Then
            Index = (Index + 1 + Images.Count) Mod Images.Count
        End If
    End Sub

    Private Sub OnAddSubordinateClicked(ByVal sender As Object, ByVal e As EventArgs)
        If Parent Is Nothing Then
            Return
        End If

        Dim newNode As New EmployeeNode(Parent)
        newNode.Bounds = New RectangleF(Bounds.Left + 10, Bounds.Bottom + 10, 80, 40)
        Parent.Nodes.Add(newNode)

        Parent.Factory.CreateDiagramLink(Me, newNode)

        Dim rootCount As Integer = 0
        For Each node As DiagramNode In Parent.Nodes
            If node.IncomingLinks.Count = 0 Then
                rootCount += 1
            End If
        Next

        ' Run a tree layout on this branch
        Dim layout As New TreeLayout()
        layout.Type = TreeLayoutType.Centered
        layout.LinkStyle = TreeLayoutLinkType.Cascading3
        layout.Direction = TreeLayoutDirections.TopToBottom
        If rootCount <= 1 Then
            layout.KeepRootPosition = True
        Else
            layout.KeepRootPosition = False
        End If
        layout.LevelDistance = 15
        layout.Arrange(Parent)

        Dim nodeBounds As RectangleF = RectangleF.Empty
        For Each node As DiagramNode In Parent.Nodes
            If nodeBounds.IsEmpty Then
                nodeBounds = node.Bounds
            Else
                nodeBounds = RectangleF.Union(nodeBounds, node.Bounds)
            End If
        Next

        Dim documentBounds As RectangleF = Parent.Bounds
        Const Threshold As Single = 5
        If nodeBounds.Left < documentBounds.Left + Threshold Then
            documentBounds.Width = (documentBounds.Right - nodeBounds.X + Threshold)
            documentBounds.X = nodeBounds.X - Threshold
        End If
        If nodeBounds.Top < documentBounds.Top + Threshold Then
            documentBounds.Height = (documentBounds.Bottom - nodeBounds.Y + Threshold)
            documentBounds.Y = nodeBounds.Y - Threshold
        End If
        If nodeBounds.Right > documentBounds.Right + Threshold Then
            documentBounds.Width = nodeBounds.Right - documentBounds.X + Threshold
        End If
        If nodeBounds.Bottom > documentBounds.Bottom + Threshold Then
            documentBounds.Height = nodeBounds.Bottom - documentBounds.Y + Threshold
        End If

        Parent.Bounds = documentBounds
    End Sub

    Private Sub OnFullNameGotFocus(ByVal sender As Object, ByVal e As EventArgs)
        If m_fullName.Text = FullNamePlaceholder Then
            m_fullName.TextColor = Color.Black
            m_fullName.Text = ""
        End If

        isFullNameFocused = True
        UpdateState(m_fullName, isFullNameFocused, isMouseOverFullName)
    End Sub

    Private Sub OnFullNameLostFocus(ByVal sender As Object, ByVal e As EventArgs)
        isFullNameFocused = False
        UpdateState(m_fullName, isFullNameFocused, isMouseOverFullName)
    End Sub

    Private Sub OnFullNameMouseMove(ByVal sender As Object, ByVal e As ComponentMouseEventArgs)
        isMouseOverFullName = m_fullName.Bounds.Contains(m_fullName.ToLocal(e.MousePosition))
        UpdateState(m_fullName, isFullNameFocused, isMouseOverFullName)
    End Sub

    Private Sub OnTextGotFocus(ByVal sender As Object, ByVal e As EventArgs)
        If m_text.Text = TextPlaceholder Then
            m_text.TextColor = Color.Black
            m_text.Text = ""
        End If

        isTextFocused = True
        UpdateState(m_text, isTextFocused, isMouseOverText)
    End Sub

    Private Sub OnTextLostFocus(ByVal sender As Object, ByVal e As EventArgs)
        isTextFocused = False
        UpdateState(m_text, isTextFocused, isMouseOverText)
    End Sub

    Private Sub OnTextMouseMove(ByVal sender As Object, ByVal e As ComponentMouseEventArgs)
        isMouseOverText = m_text.Bounds.Contains(m_text.ToLocal(e.MousePosition))
        UpdateState(m_text, isTextFocused, isMouseOverText)
    End Sub

    Private Sub UpdateState(ByVal component As EditComponent, ByVal isFocused As Boolean, ByVal isMouseOver As Boolean)
        If isFocused Then
            If isMouseOver Then
                component.Pen = New MindFusion.Drawing.Pen(Color.DarkGoldenrod)
            Else
                component.Pen = New MindFusion.Drawing.Pen(Color.Goldenrod)
            End If
            component.Brush = New MindFusion.Drawing.SolidBrush(Color.White)
        Else
            If isMouseOver Then
                component.Pen = New MindFusion.Drawing.Pen(Color.Goldenrod)
            Else
                component.Pen = New MindFusion.Drawing.Pen(Color.Transparent)
            End If
            component.Brush = New MindFusion.Drawing.SolidBrush(Color.Transparent)
        End If
    End Sub


    Public Property Stroke() As MindFusion.Drawing.Pen
        Get
            Return shape.Pen
        End Get
        Set(ByVal value As MindFusion.Drawing.Pen)
            shape.Pen = value
            If Parent IsNot Nothing Then
                Parent.Invalidate(Bounds)
            End If
        End Set
    End Property

    Public Property Fill() As MindFusion.Drawing.Brush
        Get
            Return shape.Brush
        End Get
        Set(ByVal value As MindFusion.Drawing.Brush)
            shape.Brush = value
            If Parent IsNot Nothing Then
                Parent.Invalidate(Bounds)
            End If
        End Set
    End Property

    Public Property Image() As Image
        Get
            Return m_image.Image
        End Get
        Set(ByVal value As Image)
            m_image.Image = value
            If Parent IsNot Nothing Then
                Parent.Invalidate(Bounds)
            End If
        End Set
    End Property

    Public Property Index() As Integer
        Get
            Return Images.IndexOf(Image)
        End Get
        Set(ByVal value As Integer)
            If value <> -1 Then
                Image = Images(value)
				'Title = Titles(value)
                Fill = Fills(value)
            Else
                Image = Nothing
            End If

            If Parent IsNot Nothing Then
                Parent.Invalidate(Bounds)
            End If
        End Set
    End Property

    Public Property Title() As String
        Get
            Return m_title.Text
        End Get
        Set(ByVal value As String)
            If String.IsNullOrEmpty(value) Then
                m_title.Text = " "
            Else
                m_title.Text = value
            End If
            If Parent IsNot Nothing Then
                Parent.Invalidate(Bounds)
            End If
        End Set
    End Property

    Public Property FullName() As String
        Get
            Return m_fullName.Text
        End Get
        Set(ByVal value As String)
            m_fullName.Text = value
            If Parent IsNot Nothing Then
                Parent.Invalidate(Bounds)
            End If
        End Set
    End Property

    Public Property Text() As String
        Get
            Return m_text.Text
        End Get
        Set(ByVal value As String)
            m_text.Text = value
            If Parent IsNot Nothing Then
                Parent.Invalidate(Bounds)
            End If
        End Set
    End Property


    Private Shared Images As List(Of Image)
    Private Shared Titles As List(Of String)
    Private Shared Fills As List(Of MindFusion.Drawing.Brush)

    Private Const FullNamePlaceholder As String = "<enter name>"
    Private Const TextPlaceholder As String = "<enter description>"


    Private shape As ShapeComponent
    Private m_image As ImageComponent
    Private m_title As TextComponent
    Private m_fullName As EditComponent
    Private isMouseOverFullName As Boolean
    Private isFullNameFocused As Boolean
    Private m_text As EditComponent
    Private isMouseOverText As Boolean
    Private isTextFocused As Boolean

End Class
